// HTreeCanvas : header file
//
// Created: 9/8-2007
// Last update: 12/27-2007
//
//
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2007  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/


#ifndef HTREECANVAS__H__
#define HTREECANVAS__H__

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "htreecanvas.cpp"
#endif


#include <wx/wx.h>
#include <wx/scrolwin.h>
#include <wx/print.h>
#include <viewmetrics.h>
#include <layout.h>
#include <monads.h>

class HTreeNode {
 protected:
	int m_x, m_y; // y-values grow from the bottom (0) upwards (positive)
	int m_width, m_height;
	int m_main_box_width;
	int m_least_vertical_riser_width, m_least_vertical_riser_height;
	int m_leftmost_x, m_rightmost_x;
	id_d_t m_self_id_d;
	id_d_t m_parent_id_d;
	SetOfMonads m_som;
	LayoutBoxBase *m_pBranchLabels;
	bool m_bOwnsBranchLabels;
	LayoutBoxBase *m_pMainBox;
	bool m_bOwnsMainBox;
 public:
	HTreeNode(id_d_t self_id_d,
		  id_d_t parent_id_d,
		  const SetOfMonads& som,
		  LayoutBoxBase *pBranchLabels,
		  bool bOwnsBranchLabels,
		  LayoutBoxBase *pMainBox,
		  bool bOwnsMainBox);
	virtual ~HTreeNode();
	virtual bool isTerminal(void) const { return false; };
	virtual void RecalculateExtents(wxDC *pDC, ViewMetrics *pViewMetrics);
	int getVerticalRiserX(void) const;
	int getLeastVerticalRiserHeight(void) const { return m_least_vertical_riser_height; };
	int getLeastVerticalRiserWidth(void) const { return m_least_vertical_riser_width; };
	const SetOfMonads& getSOM(void) const { return m_som; };
	id_d_t getID_D(void) const { return m_self_id_d; };
	id_d_t getParentID_D(void) const { return m_parent_id_d; };
	monad_m first(void) const { return m_som.first(); };
	monad_m last(void) const { return m_som.last(); };
	int getMainBoxWidth(void) const { return m_main_box_width; };
	int getWidth(void) const { return m_width; };
	int getHeight(void) const { return m_height; };
	int getMiddleY(int y) const { return y + m_height/2; };
	int getBottomY(int y) const { return y + m_height; };
	virtual void calculateLeastVerticalRiserDimensions(ViewMetrics *pViewMetrics);
	virtual int calculateMainHeight(void) { return 0; }; // Must be overridden
	virtual int calculateMainWidth(void) { return 0; }; // Must be overridden
	virtual void calculateDimensions(ViewMetrics *pViewMetrics);
	void setX(int x) { m_x = x; };
	void setY(int y) { m_y = y; };
	int getX(void) const { return m_x; };
	int getY(void) const { return m_y; };
	void setLeftmostX(int leftmost_x) { m_leftmost_x = leftmost_x; };
	void setRightmostX(int rightmost_x) { m_rightmost_x = rightmost_x; };
	void setXRightToLeft(int total_canvas_width);

	virtual void Draw(wxDC *pDC, int y, int yParentMiddle, int yParentBottom, int xParentLeft, int xParentRight);
};

class HTerminalNode : public HTreeNode {
 public:
	HTerminalNode(id_d_t self_id_d,
		      id_d_t parent_id_d,
		      const SetOfMonads& som,
		      LayoutBoxBase *pBranchLabels,
		      bool bOwnsBranchLabels,
		      LayoutBoxBase *pTerminalBox,
		      bool bOwnsTerminalBox);
	virtual ~HTerminalNode();
	
	virtual bool isTerminal(void) const { return true; };

	virtual int calculateMainHeight(void);
	virtual int calculateMainWidth(void);

	virtual void Draw(wxDC *pDC, int y, int yParentMiddle, int yParentBottom, int xParentLeft, int xParentRight);
};

class HNonTerminalNode : public HTreeNode {
 public:
	HNonTerminalNode(id_d_t self_id_d,
			 id_d_t parent_id_d,
			 const SetOfMonads& som,
			 LayoutBoxBase *pBranchLabels,
			 bool bOwnsBranchLabels,
			 LayoutBoxBase *pCategoryBox,
			 bool bOwnsCategoryBox);
	virtual ~HNonTerminalNode();

	virtual int calculateMainHeight(void);
	virtual int calculateMainWidth(void);

	virtual void Draw(wxDC *pDC, int y, int yParentMiddle, int yParentBottom, int xParentLeft, int xParentRight);
};


class HTreeCanvas : public wxScrolledWindow {
 protected:
	typedef std::map<id_d_t, HTreeNode*> IDD2TreeNodeMap;
	typedef std::map<monad_m, id_d_t> Monad2IDDMap;
	typedef std::map<id_d_t, Monad2IDDMap> IDD2Monad2IDDMap;
	IDD2TreeNodeMap m_objs;
	Monad2IDDMap m_terminals;
	IDD2Monad2IDDMap m_children;
	int m_width, m_height;
public:
	HTreeCanvas(wxWindow *pParent, const wxPoint& pos, const wxSize& size, const long style, bool bRightToLeft);
	virtual void OnDraw(wxDC &dc);
	virtual void DrawToClipboard(void);
	virtual void DrawToBitmap(wxBitmap **ppBitmap);
	void OnMouseEvent(wxMouseEvent& event);
	void OnSize(wxSizeEvent& event);

	void addTerminal(HTerminalNode *pNode);
	void addNonTerminal(HNonTerminalNode *pNode);
	void layout();

	void clearLayout(); // Deletes all objects...
	
	int GetWidth(void) const { return m_width; };
	int GetHeight(void) const { return m_height; };

	DECLARE_EVENT_TABLE()
public:
	int m_nMapMode;
	int m_nMagnification;
	ViewMetrics *pViewMetrics;
	bool m_bRightToLeft;
 protected:
	void addObject(HTreeNode *pNode);
	HTreeNode *getNode(id_d_t id_d);
	int getInterWordSpace() const;
	void layoutNode(id_d_t id_d);
	void setXRightToLeft(id_d_t id_d);
	const Monad2IDDMap& getRoots() const;
	const Monad2IDDMap& getChildren(id_d_t parent_id_d) const;
	void DrawNode(wxDC *pDC, id_d_t id_d, int yParentMiddle, int yParentBottom, int xParentLeft, int xParentRight);
public:
	void PrepareDC(wxDC *pDC);
	virtual void OnChangeInterlinearLine();
	virtual void OnUpdate();
	virtual void UpdateAllViews() {};  // Must be implemented.
	virtual void ChangeMagnification(int magnification);
	int GetMagnification(void) const { return m_nMagnification; };
	ViewMetrics *GetViewMetrics(void) { return pViewMetrics; };
	virtual void InitializeViewMetrics(); // Call this AFTER initializing it!
	bool UpdateViewMetricsViewWidth(void);
	virtual void HandleButtonDown(wxMouseEvent& event, wxPoint point, bool RightButton);
	void ShowPopup(wxMenu *menuPopUp, wxPoint point);
 protected:
	
	virtual ~HTreeCanvas();
	void GetClipBox(wxDC *pDC, wxRect& clipRect);
	void adjustHeights(wxDC* pDC);
};




#endif

